@@ -24,6 +24,7 @@ from functools import lru_cache
 import attr
 import asyncio
 import aiohttp
+from urllib.parse import unquote
 
 from bs4 import BeautifulSoup
 
@@ -58,7 +59,7 @@ class AsyncIterablePackages:
 
     async def fetch_data(self) -> Optional[str]:  # Ignore PyDocStyleBear
         """Fetch data."""
-        await asyncio.sleep(0.1)  # Other coros get to run
+        await asyncio.sleep(0)  # Other coros get to run
 
         if len(self.packages) == 0:
             return None
@@ -88,7 +89,7 @@ class AsyncIterableVersions:
 
     async def fetch_data(self) -> Optional[str]:  # Ignore PyDocStyleBear
         """Fetch data."""
-        await asyncio.sleep(0.1)  # Other coros get to run
+        await asyncio.sleep(0)  # Other coros get to run
 
         if len(self.versions) == 0:
             return None
@@ -118,7 +119,7 @@ class AsyncIterableArtifacts:
 
     async def fetch_data(self) -> Optional[Tuple]:  # Ignore PyDocStyleBear
         """Fetch data."""
-        await asyncio.sleep(0.1)  # Other coros get to run
+        await asyncio.sleep(0)  # Other coros get to run
 
         if len(self.artifacts) == 0:
             return None
@@ -139,14 +140,17 @@ class AIOSource(Source):
         _LOGGER.debug("Gathering package version information from Warehouse API: %r", url)
 
         async with aiohttp.ClientSession(raise_for_status=True) as session:
-            async with session.get(url) as response:
-                if response.status == 404:
+            try:
+                async with session.get(url) as response:
+                    return await response.json()
+            except aiohttp.ClientResponseError as exc:
+                if exc.status == 404:
                     raise NotFoundError(
                         f"Package {package_name} in version {package_version} not "
                         f"found on warehouse {self.url} ({self.name})"
                     )
 
-                return await response.json()
+                raise
 
     async def _warehouse_get_package_hashes(  # type: ignore
         self, package_name: str, package_version: str, with_included_files: bool = False
@@ -172,11 +176,14 @@ class AIOSource(Source):
         _LOGGER.debug("Gathering package information from Warehouse API: %r", url)
 
         async with aiohttp.ClientSession(raise_for_status=True) as session:
-            async with session.get(url) as response:
-                if response.status == 404:
+            try:
+                async with session.get(url) as response:
+                    return await response.json()
+            except aiohttp.ClientResponseError as exc:
+                if exc.status == 404:
                     raise NotFoundError(f"Package {package_name} not found on warehouse {self.url} ({self.name})")
 
-                return await response.json()
+                raise
 
     async def _simple_repository_list_versions(self, package_name: str) -> List:  # type: ignore
         """List versions of package available on a simple repository."""
@@ -199,16 +206,18 @@ class AIOSource(Source):
         _LOGGER.debug("Discovering package %r artifacts from %r", package_name, url)
 
         async with aiohttp.ClientSession(raise_for_status=True) as session:
-            async with session.get(url) as response:
-                if response.status == 404:
+            try:
+                async with session.get(url) as response:
+                    text = await response.text()
+                    soup = BeautifulSoup(text, "lxml")
+                    links = soup.find_all("a")
+            except aiohttp.ClientResponseError as exc:
+                if exc.status == 404:
                     raise NotFoundError(
                         f"Package {package_name} is not present on index {self.url} (index {self.name})"
                     )
 
-                text = await response.text()
-                soup = BeautifulSoup(text, "lxml")
-
-                links = soup.find_all("a")
+                raise
 
         artifacts = []  # List[Tuple[str,str]]
         for link in links:
@@ -231,6 +240,9 @@ class AIOSource(Source):
             if not artifact_url.startswith(("http://", "https://")):
                 artifact_url = url + f"/{artifact_name}"
 
+            # Decode characters in link retrieved
+            artifact_name = unquote(artifact_name)
+
             artifacts.append((artifact_name, artifact_url))
 
         return AsyncIterableArtifacts(artifacts)
@@ -286,10 +298,13 @@ class AIOSource(Source):
         possible_continuations = [".win32", ".tar.gz", ".whl", ".zip", ".exe", ".egg", "-"]
         async for artifact_name, artifact_url in await self._simple_repository_list_artifacts(package_name):
             # Convert all artifact names to lowercase - as a shortcut we simply convert everything to lowercase.
-            artifact_name.lower()
+            artifact_name = artifact_name.lower()
 
             for i in possible_continuations:
-                if artifact_name.startswith(f"{package_name}-{package_version}{i}"):
+                if (
+                    artifact_name.startswith(f"{package_name}-{package_version}")
+                    or artifact_name.startswith(f"{package_name.replace('-', '_')}-{package_version}")
+                ) and artifact_name.endswith(i):
                     break
             else:
                 _LOGGER.debug(
